<?php

namespace Drupal\simply_signups\Form;

use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Form\FormBase;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\filter\FilterFormatInterface;
use Drupal\filter\Plugin\FilterInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\simply_signups\Utility\SimplySignupsUtility;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Implements a signup form.
 */
class SimplySignupsForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'simply_signups_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $nid = NULL) {
    $db = \Drupal::database();
    $query = $db->select('simply_signups_fields', 'p');
    $query->fields('p');
    $query->orderBy('weight');
    $query->condition('nid', $nid, '=');
    $results = $query->execute()->fetchAll();
    $fieldCount = $query->countQuery()->execute()->fetchField();
    $form['#attached']['library'][] = 'simply_signups/styles';
    $form['#attributes'] = [
      'class' => ['simply-signups-rsvp-form', 'simply-signups-form'],
    ];
    if ($fieldCount > 0) {
      $form['signup_fieldset'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Signup for this event'),
      ];
      $results = $query->execute()->fetchAll();
      foreach ($results as $row) {
        $field = unserialize($row->field);
        if (($field['#type'] == 'select') OR ($field['#type'] == 'radios') OR ($field['#type'] == 'checkboxes')) {
          $form['signup_fieldset'][$field['#title']] = [
            '#type' => $field['#type'],
            '#title' => $row->name,
            '#options' => $field['#options'],
            '#description' => (isset($field['#description'])) ? $field['#description'] : '',
            '#default_value' => (isset($field['#default_value'])) ? [$field['#default_value']] : NULL,
            '#required' => $field['#required'],
          ];
        }
        else {
          $type = $field['#type'];
          $form['signup_fieldset'][$field['#title']] = [
            '#type' => $field['#type'],
            '#title' => $row->name,
            '#description' => (isset($field['#description'])) ? $field['#description'] : '',
            '#default_value' => (isset($field['#default_value'])) ? $field['#default_value'] : '',
            '#required' => $field['#required'],
          ];
        }
      }
      $form['signup_fieldset']['nid'] = [
        '#type' => 'hidden',
        '#value' => $nid,
      ];
      $form['signup_fieldset']['actions'] = [
        '#type' => 'actions',
      ];
      $form['signup_fieldset']['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Signup'),
        '#attributes' => [
          'class' => [
            'btn-primary',
          ],
        ],
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $node = $values['nid'];
    $numberOfAttending = SimplySignupsUtility::get_number_of_attending($id);
    $maxAttending = SimplySignupsUtility::get_max_attending($nid);
    $totalAttending = ($values['number_attending'] + $numberOfAttending);
    if ($maxAttending > 0 AND $totalAttending > $maxAttending) {
      $form_state->setErrorByName('number_attending', t('Sorry but the number of attending that you designated would be over the maximum attending allowed for this event.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $nid = $values['nid'];
    $node = Node::load($nid);
    $title = $node->getTitle();
    $requestTime = \Drupal::time()->getCurrentTime();
    $date = format_date($requestTime, 'custom', 'm/d/Y - h:i a');
    $options = ['absolute' => TRUE];
    $url = Url::fromRoute('entity.node.canonical', ['node' => $nid], $options);
    $nodeLink = Link::fromTextAndUrl('View the event', $url)->toString();
    $options = ['absolute' => TRUE];
    $url = Url::fromRoute('simply_signups.nodes', ['node' => $nid], $options);
    $link = Link::fromTextAndUrl('View all submissions', $url)->toString();
    foreach ($values as $key => $value) {
      if ($key != 'submit' AND $key != 'form_build_id' AND $key != 'form_token' AND $key != 'form_id' AND $key != 'op' AND isset($form['signup_fieldset'][$key]['#title'])) {
        $fields['fields'][$key]['type'] = $form['signup_fieldset'][$key]['#type'];
        $fields['fields'][$key]['value'] = $value;
        $fields['fields'][$key]['title'] = $form['signup_fieldset'][$key]['#title'];
      }
    }
    unset($fields['fields']['submit']);
    unset($fields['fields']['form_build_id']);
    unset($fields['fields']['form_token']);
    unset($fields['fields']['form_id']);
    unset($fields['fields']['op']);
    $numberAttendingFlag = (isset($fields['fields']['number_attending'])) ? 1 : 0;
    $fields['fields']['number_attending']['type'] = 'select';
    $fields['fields']['number_attending']['value'] = (isset($fields['fields']['number_attending']['value'])) ? $fields['fields']['number_attending']['value'] : 1;
    $fields['fields']['number_attending']['title'] = (isset($fields['fields']['number_attending']['title'])) ? $fields['fields']['number_attending']['title'] : '# Attending';
    $numberAttending = $fields['fields']['number_attending']['value'];
    if ($numberAttendingFlag == 0) {
      unset($fields['fields']['number_attending']);
    }
    $row = [
      'nid' => $nid,
      'fields' => serialize($fields['fields']),
      'attending' => $numberAttending,
      'status' => 0,
      'created' => $requestTime,
      'updated' => $requestTime,
    ];
    db_insert('simply_signups_data')->fields($row)->execute();
    $config = \Drupal::config('simply_signups.config');
    $db = \Drupal::database();
    $query = $db->select('simply_signups_settings', 'p');
    $query->fields('p');
    $query->condition('nid', $nid, '=');
    $results = $query->execute()->fetchAll();
    $system = \Drupal::config('system.site');
    $siteName = $system->get('name');
    $host = \Drupal::request()->getHost();
    foreach ($results as $row) {
      $nid = $row->nid;
      $startDate = $row->start_date;
      $endDate = $row->end_date;
      $altPath = $row->path;
      $isValidAbsolute = UrlHelper::isValid($altPath, $absolute = TRUE);
      $maxSignups = $row->max_signups;
      $status = $row->status;
      $adminSendMail = $row->admin_send_mail;
      $adminMail = (valid_email_address($row->admin_mail)) ? $row->admin_mail : 'webmaster@'.$host;
      $adminSubject = (!empty($row->admin_subject)) ? $row->admin_subject : 'RSVP for: [node:title] - '. $date;
      $adminMessage = (!empty($row->admin_message)) ? $row->admin_message : '<p>A RSVP has been submitted for an event below.</p><h3>[node:title]</h3><p>[node:url]<br />'. $link. '</p><p>Submitted on: '. format_date($requestTime, 'custom', 'm/d/Y - h:i a'). '</p>';
      $adminFormat = $row->admin_format;
      $clientSendMail = $row->client_send_mail;
      $clientSubject = (!empty($row->client_subject)) ? $row->client_subject : 'RSVP Confirmation: [node:title] - '. $date;
      $clientMessage = (!empty($row->send_message)) ? $row->send_message : "<h3>Thank you for your RSVP!</h3><p>We're happy to see that you're interested in coming to our event, [node:title]!</p><p>You can find event details at [node:url]</p><p>If you are no longer able to attend the event, that's fine! We kindly ask that you let us know by contacting us at: [site:mail].</p><p>Thank you! We look forward to seeing you soon.</p><p>Sincerely,<br />[site:name]</p>";
      $clientFormat = $row->client_format;
      $created = $row->created;
      $updated = $row->updated;
    }
    $send = new \Drupal\Core\Mail\Plugin\Mail\PhpMail();
    $from = $adminMail;

    $message['headers'] = [
      'Content-Type' => 'text/html; charset=UTF-8; format=flowed; delsp=yes',
      'MIME-Version' => '1.0',
      'reply-to' => $from,
      'from' => $siteName. ' <'.$from.'>',
    ];
    $message['format'] = 'text/html';
    foreach ($fields['fields'] as $key => $value) {
      if (valid_email_address($value['value'])) {
        $to = $value['value'];
      }
    }
    $token_service = \Drupal::token();
    if ($clientSendMail == 1) {
      $clientTokenSubject = $token_service->replace($clientSubject,['node' => $node ]);
      $clientTokenMessage = $token_service->replace($clientMessage,['node' => $node ]);
      $clientTokenSubject = check_markup($clientTokenSubject, $clientFormat);
      $clientTokenMessage = check_markup($clientTokenMessage, $clientFormat);
      $clientSubject = $clientTokenSubject;
      $clientMessage = $clientTokenMessage;
      if (valid_email_address($to)) {
        $message['to'] = $to;
        $message['subject'] = $clientSubject;
        $message['body'] = $clientMessage;
        $message['body'] = Markup::create($message['body']);
        //$send->format($message);
        $result = $send->mail($message);
        if ($result !== true) {
          drupal_set_message('Unable to send mail please contact the webmaster.', $type = 'error');
        }
      }
    }
    if ($adminSendMail == 1) {
      $adminTokenMail = $token_service->replace($adminMail,['node' => $node ]);
      $adminMail = $adminTokenMail;
      $adminTokenSubject = $token_service->replace($adminSubject,['node' => $node ]);
      $adminTokenMessage = $token_service->replace($adminMessage,['node' => $node ]);
      $adminTokenSubject = check_markup($adminTokenSubject, $adminFormat);
      $adminTokenMessage = check_markup($adminTokenMessage, $adminFormat);
      if (valid_email_address($adminMail)) {
        $adminSubject = $adminTokenSubject;
        $adminMessage = $adminTokenMessage;
        $message['to'] = $adminMail;
        $message['subject'] = $adminSubject;
        $message['body'] = $adminMessage;
        $message['body'] = Markup::create($message['body']);
        //$send->format($message);
        $result = $send->mail($message);
        if ($result !== true) {
          drupal_set_message('Unable to send mail please contact the webmaster.', $type = 'error');
        }
      }
    }
    if ($isValidAbsolute == 1) {
      $altUrl = Url::fromUri($altPath)->toString();
      $response = new TrustedRedirectResponse($altUrl);
      $form_state->setResponse($response);
    }
    drupal_set_message($this->t('Your signup has been submitted successsfully.'));
  }
}